#include <afxwin.h>
#include <string.h>	// for strcpy and strcat
#include <stdlib.h>	// for atoi
#include <ctype.h>	// for isdigit and isalpha
#include "bapp_ids.h"
#include "structs.h"
#include "tbtrieve.h"
#include "btrapp.h"
#include "btrdlg.h"

#define BTR_INT     0x7B		    /* Btrieve interrupt vector */
#define BTR_OFFSET  0x33	       /* Btrieve offset within segment */

extern "C" { int DoneBtrieve(); };
extern "C" { int InitBtrieve(HINSTANCE hInst, unsigned RecordLeng); };
//extern "C" { int IsBtrieveLoaded(); };

#ifdef __BORLANDC__
int IsBtrieveLoaded(void)
{
  asm {
    mov ax,0200h
    mov bl,BTR_INT
    int 31h
    jnc NoError
  }
  return 0;
NoError:
  asm {
    mov ax,BTR_OFFSET
    cmp dx,ax
    je	BtrLoaded
  }
  return 0;
BtrLoaded:
  return 1;
}
#endif
#ifdef __WATCOMC__
int IsBtrieveLoaded(void);
#pragma aux IsBtrieveLoaded = \
	   "mov ax,0200h"	   \
	   "mov bl,7Bh" 	   \
	   "int 31h"		   \
	   "jnc NoError"	   \
	   "mov ax,0"		   \
	   "jmp leav"		   \
  "NoError: mov ax,33h" 	   \
	   "cmp dx,ax"		   \
	   "je	BtrLoaded"	   \
	   "mov ax,0"		   \
	   "jmp leav"		   \
"BtrLoaded: mov ax,1"		   \
     "leav: nop"		   \
	value	[ax]  \
	modify	[bx dx];
#endif

char CitysDBFileName[] = "citys.dat";
char TargetsDBFileName[] = "targets.dat";
char PlacesDBFileName[] = "places.dat";
char TrainsDBFileName[] = "trains.dat";
char StatesDBFileName[] = "states.dat";
char DivisionDBFileName[] = "division.dat";
char BtripDBFileName[] = "btrip.dat";
/*
char CitysDBFileName[] = "y:\\netware\\citys.dat";
char TargetsDBFileName[] = "y:\\netware\\targets.dat";
char PlacesDBFileName[] = "y:\\netware\\places.dat";
char TrainsDBFileName[] = "y:\\netware\\trains.dat";
char StatesDBFileName[] = "y:\\netware\\states.dat";
char DivisionDBFileName[] = "y:\\netware\\division.dat";
char BtripDBFileName[] = "y:\\netware\\btrip.dat";
*/
char Owner[] = "";

CBtripApp NEAR theApp;

BEGIN_MESSAGE_MAP(CBtripApp, CWinApp)
	ON_COMMAND( IDM_GETKOMAND, OnGetKomand )
	ON_COMMAND( IDM_EDITDIVISION, OnDivisionChange )
	ON_COMMAND( IDM_CITY, OnCityChange )
	ON_COMMAND( IDM_TARGET, OnTargetChange )
	ON_COMMAND( IDM_PLACE, OnPlaceChange )
	ON_COMMAND( IDM_TRAIN, OnTrainChange )
	ON_COMMAND( IDM_STATES, OnStatesChange )
	ON_COMMAND( IDM_VIEWKOMAND, OnGeneralView )
	ON_COMMAND( IDM_VIEWDEBTORS, OnViewDebtors )
	ON_COMMAND( IDM_GETPREP, OnGetPrepayment )
	ON_COMMAND( IDM_ABOUT, OnAppAbout)
END_MESSAGE_MAP()

BOOL CBtripApp::InitInstance()
{
	if( IsBtrieveLoaded()) {
	  if( InitBtrieve( m_hInstance, 4096)) {
	    AfxMessageBox( "  BTRIEVE", MB_OK | MB_ICONSTOP);
	    return FALSE;
	  }
	}
	else {
	  AfxMessageBox( "    BTRIEVE", MB_OK | MB_ICONSTOP);
	  return FALSE;
	}
	SetDialogBkColor();
	m_pMainWnd = new CMainFrame;
	((CFrameWnd*)m_pMainWnd)->LoadFrame(IDR_MAINFRAME);
	m_pMainWnd->ShowWindow(m_nCmdShow);
	// enable file manager drag/drop and DDE Execute open
//	  m_pMainWnd->DragAcceptFiles();
//	  EnableShellOpen();
//	  RegisterShellFileTypes();
	return TRUE;
};

int CBtripApp::ExitInstance()
{
  if( DoneBtrieve() == 0)
    return CWinApp::ExitInstance();
  else {
    if( AfxMessageBox( "     .  ?",
		       MB_ICONHAND | MB_YESNO) == IDNO)
       return 1;
    }
  return 0;
};

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)
BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT buttons[] =
{
	IDM_GETKOMAND, IDM_EDITDIVISION, ID_SEPARATOR, IDM_VIEWKOMAND,
	IDM_VIEWDEBTORS, IDM_GETPREP, ID_SEPARATOR, ID_APP_EXIT
};

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	SIZE sB = { 26, 26}, sI = { 20, 20};

	CFrameWnd::OnCreate(lpCreateStruct);
	m_ToolBar.SetSizes(sB, sI);
	return ((m_ToolBar.Create(this) &&
		m_ToolBar.LoadBitmap(IDR_MAINFRAME) &&
		m_ToolBar.SetButtons(buttons, sizeof(buttons)/sizeof(UINT)))
	  ? 0 : -1);
}

void CBtripApp::OnAppAbout()
{
	CDialog(IDD_ABOUTBOX).DoModal();
}

void CBtripApp::OnGetKomand()
{
	CGetBtripDlg(IDD_GETBTRIP).DoModal();
}

void CBtripApp::OnGetPrepayment()
{
  Exchange Data;
  if( CAttributeDlg(IDD_ATTRDIALOG, &Data).DoModal() == IDOK)
    if( CSearchDlg(IDD_SEARCHDIALOG, &Data).DoModal() == IDCANCEL)
      return;
}

void CBtripApp::OnDivisionChange()
{
  CSwitchDlg(IDD_SWITCHLIST).DoModal();
}

void CBtripApp::OnCityChange()
{
  CUpdateDlg(IDD_UPDATELIST, CitysDBFileName,"").DoModal();
}

void CBtripApp::OnTargetChange()
{
  CUpdateDlg(IDD_UPDATELIST, TargetsDBFileName,"").DoModal();
}

void CBtripApp::OnPlaceChange()
{
  CUpdateDlg(IDD_UPDATELIST, PlacesDBFileName," ").DoModal();
}

void CBtripApp::OnTrainChange()
{
  CUpdateDlg(IDD_UPDATELIST, TrainsDBFileName," ").DoModal();
}

void CBtripApp::OnStatesChange()
{
  CUpdateDlg(IDD_UPDATELIST, StatesDBFileName," ").DoModal();
}

void CBtripApp::OnViewDebtors()
{
  CGeneralViewDlg(IDD_VIEWDIALOG, Debtors).DoModal();
}

void CBtripApp::OnGeneralView()
{
  Exchange Data;
  if( CSelectDlg(IDD_SELECTDIALOG,&Data).DoModal() == IDOK)
      CGeneralViewDlg(IDD_VIEWDIALOG, All, Data.Division).DoModal();
}
