#include <afxwin.h>
#include <ctype.h>      // for isdigit and isalpha
#include "bapp_ids.h"
#include "structs.h"
#include "tbtrieve.h"
#include "btrapp.h"
#include "btrdlg.h"

enum CType { LBox, CBox };

extern char CitysDBFileName[];
extern char TargetsDBFileName[];
extern char PlacesDBFileName[];
extern char TrainsDBFileName[];
extern char StatesDBFileName[];
extern char DivisionDBFileName[];
extern char BtripDBFileName[];
extern char Owner[];

CBusinesTrip BusinesTrip;

/*--------------------------------------------------*/
/* Validate Name.        			    */
/*--------------------------------------------------*/
BOOL ValidName(char *Name)
{
  for (int i=0; i < strlen(Name); i++)
	 if (! IsCharAlpha(Name[i]) && Name[i] != ' ' && Name[i] != '.' )
	 {
           AfxMessageBox(" ",/*" ",*/ MB_OK);
           return FALSE;
	 }
  return TRUE;
}

/*--------------------------------------------------*/
/* Validate Numeric.        			    */
/*--------------------------------------------------*/
BOOL ValidNumeric(char far *Number)
{
  for (int i=0; i < lstrlen(Number); i++)
	 if (! isdigit(Number[i]))
	 {
            AfxMessageBox(" ",/*" ",*/ MB_OK);
            return FALSE;
	 }
  return TRUE;
}

/*--------------------------------------------------*/
/* Validate Date.        			    */
/*--------------------------------------------------*/
BOOL ValidDate(char *CheckDate, Date *PNewDate)
{
  int Len = strlen(CheckDate);
  char *Digit;
  BOOL Valid  = TRUE;;

  if ((Len > 10) && strchr(CheckDate,'.') == NULL)
     Valid = FALSE;
  else
  {
    Digit = strtok( CheckDate,".");
    if ( Digit != NULL && strlen(Digit) <= 2)
      PNewDate->Day = atoi(Digit);
    else
      Valid = FALSE;
    Digit = strtok( NULL,".");
    if ( Valid && Digit != NULL && strlen(Digit) <= 2)
      PNewDate->Month = atoi(Digit);
    else
      Valid = FALSE;
    Digit = strtok( NULL,".");
    if ( Valid && Digit != NULL && strlen(Digit) <= 4)
      PNewDate->Year = atoi(Digit);
	 else
      Valid = FALSE;
  }
  if ( !Valid ){
    AfxMessageBox(" ",/*" ",*/ MB_OK);
    PNewDate->Day = 0;
    PNewDate->Month = 0;
    PNewDate->Year = 0;
  }
  return Valid;
}

void ReadFile(char *DBFileName, CWnd *ReadList, CType type)
{
  struct Data {
    char Name[MAXSTRLEN];
  } DataRec;
  int Status;
  int DataSize = sizeof(DataRec);
  TBtrieve *PBtrFile;

  PBtrFile = new TBtrieve( DBFileName);
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0) {
    if((Status = PBtrFile->StepFirst((char *) &DataRec, &DataSize)) == 0)
      while(Status == 0) {
        if(type == LBox)
          ((CListBox *) ReadList) -> AddString(DataRec.Name);
        else
          ((CComboBox *) ReadList) -> AddString(DataRec.Name);
        DataSize = sizeof(DataRec);
	Status = PBtrFile->StepNext((char *) &DataRec, &DataSize);
      }
  }
  else
    ::MessageBox(NULL, "   "," ", MB_OK);
  PBtrFile->Close();
  delete PBtrFile;
}

/*--------------------------------------------------*/
/* CGetBtripDlg implementations:                    */
/*--------------------------------------------------*/
CGetBtripDlg::CGetBtripDlg(UINT Idd)
                 : CDialog(Idd)
{
  /*
  e1 = new CEdit(this, IDC_NAMEEDIT, MAXNAMELEN);
  e2 = new CEdit(this, IDC_FORNAMEEDIT, MAXNAMELEN);
  e3 = new CEdit(this, IDC_SUBNAMEEDIT, MAXNAMELEN);
  CityList = new CComboBox();
  CityList -> Create( NULL, 0, NULL, IDC_CITYLIST);
  TargetList = new CComboBox();
  TargetList -> Create( NULL, 0, NULL, IDC_TARGETLIST);
  PlaceList = new CComboBox();
  PlaceList -> Create( NULL, 0, NULL, IDC_PLACELIST);
  TrainsList = new CComboBox();
  TrainsList -> Create( NULL, 0, NULL, IDC_TRAINSLIST);
  StatList = new CComboBox();
  StatList -> Create( NULL, 0, NULL, IDC_STATLIST);
  DvsnList = new CComboBox();
  DvsnList -> Create( NULL, 0, NULL, IDC_DIVISIONLIST);
  */
  //PBusinesTrip = (CBusinesTrip *) &BusinesTrip;
  /* PBusinesTrip = (CBusinesTrip *)
		  &(((TKomandWindow *)Parent)->BusinesTrip); */
  //memset(PBusinesTrip, 0x0, sizeof(struct CBusinesTrip));
}

CGetBtripDlg::~CGetBtripDlg()
{
  DivisionList *Deleted, *Current;
  for( Current = Home; Current != NULL; ) {
    Deleted = Current;
    Current = Current -> Next;
    delete Deleted;
  }
  /*
  delete e1; delete e2; delete e3;
  delete DvsnList; delete CityList; delete TargetList;
  delete PlaceList; delete TrainsList; delete StatList;
  */
}

void CGetBtripDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_NAMEEDIT, e1);
  DDX_Control( pDX, IDC_FORNAMEEDIT, e2);
  DDX_Control( pDX, IDC_SUBNAMEEDIT, e3);
  DDX_Control( pDX, IDC_PNUMBEREDIT, p1);
  DDX_Control( pDX, IDC_CITYLIST, CityList);
  DDX_Control( pDX, IDC_TARGETLIST, TargetList);
  DDX_Control( pDX, IDC_PLACELIST, PlaceList);
  DDX_Control( pDX, IDC_TRAINSLIST, TrainsList);
  DDX_Control( pDX, IDC_STATLIST, StatList);
  DDX_Control( pDX, IDC_DIVISIONLIST, DvsnList);
  /*
  DDX_Text( pDX, IDC_NAMEEDIT, BusinesTrip.Name);
  DDX_Text( pDX, IDC_FORNAMEEDIT, BusinesTrip.ForName);
  DDX_Text( pDX, IDC_SUBNAMEEDIT, BusinesTrip.SubName);
  DDX_Text( pDX, IDC_PNUMBEREDIT, BusinesTrip.PNumber);
  DDX_Text( pDX, IDC_UDATEEDIT, UDateString);
  DDX_Text( pDX, IDC_PDATEEDIT, PDateString);
  DDX_CBString( pDX, IDC_CITYLIST, BusinesTrip.City);
  DDX_CBString( pDX, IDC_TARGETLIST, BusinesTrip.Target);
  DDX_CBString( pDX, IDC_PLACELIST, BusinesTrip.Place);
  DDX_CBString( pDX, IDC_TRAINSLIST, BusinesTrip.Trains);
  DDX_CBString( pDX, IDC_STATLIST, BusinesTrip.States);
  //DDX_CBString( pDX, IDC_DIVISIONLIST, BusinesTrip.SubName);
  */
}

BOOL CGetBtripDlg::OnInitDialog()
{
  int DataSize = sizeof(struct Division);
  int Status = 0;
  long Key = 0;
  DivisionList *Active, *New = (DivisionList *) new DivisionList;
  TBtrieve *PBtrFile = new TBtrieve(DivisionDBFileName);

  CDialog::OnInitDialog();
  ReadFile( CitysDBFileName, (CWnd *) &CityList, CBox);
  ReadFile( TargetsDBFileName, (CWnd *) &TargetList, CBox);
  ReadFile( PlacesDBFileName, (CWnd *) &PlaceList, CBox);
  ReadFile( TrainsDBFileName, (CWnd *) &TrainsList, CBox);
  ReadFile( StatesDBFileName, (CWnd *) &StatList, CBox);
  Active = New;
  Active -> Next = NULL;
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0)
	 if((Status = PBtrFile->GetFirst((char *)&Active->Item, &DataSize,
					 (char *)&Key, 0)) == 0) {
                DvsnList.AddString(New->Item.Name);
		Home = Active = New;
      while(Status == 0) {
	New = new DivisionList;
	New -> Next = NULL;
	DataSize = sizeof(struct Division);
	Status = PBtrFile->GetNext((char *)&New->Item, &DataSize,
				   (char *)&Key, 0);
	if(Status == 0) {
	  Active -> Next = End = New;
          DvsnList.AddString(New->Item.Name);
	  Active = Active -> Next;
	}
	else
	  delete New;
      }
      End = Active;
    }
  else
    MessageBox("   "," ", MB_OK);
  PBtrFile->Close();
  delete PBtrFile;
  return TRUE;
}

/*--------------------------------------------------*/
/* Retrieve data.                                   */
/*--------------------------------------------------*/
void CGetBtripDlg::DoFillBuffer()
{
   GetDlgItemText( IDC_NAMEEDIT, BusinesTrip.Name, MAXNAMELEN);
   GetDlgItemText( IDC_FORNAMEEDIT, BusinesTrip.ForName, MAXNAMELEN);
   GetDlgItemText( IDC_SUBNAMEEDIT, BusinesTrip.SubName, MAXNAMELEN);
   GetDlgItemText( IDC_PNUMBEREDIT, BusinesTrip.PNumber, MAXNAMELEN);
   CityList.GetLBText( CityList.GetCurSel(), BusinesTrip.City);
   TargetList.GetLBText( TargetList.GetCurSel(), BusinesTrip.Target);
   PlaceList.GetLBText( PlaceList.GetCurSel(), BusinesTrip.Place);
   TrainsList.GetLBText( TrainsList.GetCurSel(), BusinesTrip.Trains);
   StatList.GetLBText( StatList.GetCurSel(), BusinesTrip.States);
   GetDlgItemText( IDC_UDATEEDIT, UDateString, MAXNAMELEN);
   GetDlgItemText( IDC_PDATEEDIT, PDateString, MAXNAMELEN);
}

void CGetBtripDlg::OnOK()
{
  UpdateData(TRUE);
  DoFillBuffer();

  /* Make sure all input fields have been filled in.*/
  if ((BusinesTrip.Name[0]) && (BusinesTrip.ForName[0]) &&
      (BusinesTrip.SubName[0]) && (BusinesTrip.City[0]) &&
      (BusinesTrip.Target[0]) && (BusinesTrip.Place[0]) &&
      (BusinesTrip.Trains[0]) && (BusinesTrip.PNumber[0]))
  {
    char Name[MAXDNAMELEN];
    // Make sure all input fields have valid data
    if (! ValidNames() || ! ValidDates())
      return;
    DvsnList.GetWindowText( Name, MAXDNAMELEN);
    for(DivisionList *Checked = Home; Checked != NULL; Checked = Checked -> Next)
      if( strcmp(Name, Checked->Item.Name) == 0) {
        BusinesTrip.Division = Checked->Item.Code;
	break;
      }
    if( Checked == NULL) {
      BusinesTrip.Division = 0;
      MessageBox( "    "," ", MB_OK);
      return;
    }
    Date BlankDate = { 0, 0, 0};
    TBtrieve *PBusTrip = new TBtrieve(BtripDBFileName);
    int Status, RecordSize;
    unsigned Key = 0;
    BusinesTrip.FUDate = BlankDate;
    BusinesTrip.FPDate = BlankDate;
    BusinesTrip.SMoney = 0L;
    BusinesTrip.IsAccount = 0;
    BusinesTrip.IsDebt = 0;
    BusinesTrip.ADate  = BlankDate;
    BusinesTrip.PrDate = BlankDate;
    BusinesTrip.PrMoney = 0L;
    BusinesTrip.IsPrep = 0;
    BusinesTrip.Reserved = 0;
    if((Status = PBusTrip->Open(Owner, omNormal)) == 0) {
      RecordSize = sizeof(struct CBusinesTrip);
      Status = PBusTrip->Insert((char *)PBusinesTrip, &RecordSize,
				(char *)&Key, 0);
    }
    PBusTrip->Close();
    if( Status != 0) {
      MessageBox("      ","",
		 MB_OK | MB_ICONSTOP);
      return;
    }
    else {
      MessageBox("   ","", MB_OK);
    }
  }
  else {
    MessageBox("   "," ", MB_OK);
    return;
  }
  CDialog::OnOK();
}

BOOL CGetBtripDlg::ValidNames()
{
  return (ValidName(BusinesTrip.Name) == TRUE) ?
         ((ValidName(BusinesTrip.ForName) == TRUE) ?
         ((ValidName(BusinesTrip.SubName) == TRUE) ?
         ((ValidName(BusinesTrip.City) == TRUE) ?
         ((ValidName(BusinesTrip.Target) == TRUE) ?
         ((ValidName(BusinesTrip.Place) == TRUE) ?
         ((ValidName(BusinesTrip.Trains) == TRUE) ? TRUE : FALSE)
	 : FALSE)
	 : FALSE)
	 : FALSE)
	 : FALSE)
	 : FALSE)
	 : FALSE;
}

BOOL CGetBtripDlg::ValidDates()
{
  return (ValidDate( UDateString, &BusinesTrip.UDate) == TRUE) ?
         ((ValidDate( PDateString, &BusinesTrip.PDate) == TRUE) ?
	 TRUE : FALSE) : FALSE;
}

/*--------------------------------------------------*/
/* CSwitchDlg implementations:                      */
/*--------------------------------------------------*/
CSwitchDlg::CSwitchDlg(UINT Idd)
                 : CDialog(Idd)
{
  Home = End = NULL;
//  SwitchList = new CListBox();
//  SwitchList.Create( NULL, 0, NULL, IDC_SWITCHLIST);
}

CSwitchDlg::~CSwitchDlg()
{
  DivisionList *Deleted, *Current;
  for( Current = Home; Current != NULL; ) {
    Deleted = Current;
    Current = Current -> Next;
    delete Deleted;
  }
  //delete SwitchList;
}

void CSwitchDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_SWITCHLIST, SwitchList);
}

BOOL CSwitchDlg::OnInitDialog()
{
  int DataSize = sizeof(struct Division);
  int Status = 0;
  long Key = 0;
  DivisionList *Active, *New = new DivisionList;
  TBtrieve *PBtrFile = new TBtrieve(DivisionDBFileName);

  CDialog::OnInitDialog();
  Active = New;
  Active -> Next = NULL;
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0)
    if((Status = PBtrFile->GetFirst((char *)&Active->Item, &DataSize,
				    (char *)&Key, 0)) == 0) {
      SwitchList.AddString(New->Item.Name);
      Home = Active = New;
      while(Status == 0) {
	New = new DivisionList;
	New -> Next = NULL;
	DataSize = sizeof(struct Division);
	Status = PBtrFile->GetNext((char *)&New->Item, &DataSize,
				   (char *)&Key, 0);
	if(Status == 0) {
	  Active -> Next = End = New;
          SwitchList.AddString(New->Item.Name);
	  Active = Active -> Next;
	}
	else
	  delete New;
      }
      End = Active;
    }
  else
    MessageBox("   "," ", MB_OK);
  PBtrFile->Close();
  delete PBtrFile;
  return TRUE;
}

void CSwitchDlg::AddItem( )
{
  int Status = 0;
  long Key;
  char KeyString[MAXDATELEN] = "";

  DivisionList *NewItem = new DivisionList;
  int DataSize = sizeof(struct Division);
  GetDlgItemText(IDC_ITEMEDIT, NewItem->Item.Name, MAXDNAMELEN);
  for(DivisionList *Check = Home; Check != NULL; Check = Check -> Next)
    if( strcmp(NewItem->Item.Name, Check->Item.Name) == 0) {
      delete NewItem;
      MessageBox("    "," ", MB_OK);
      return;
    }
  if( CCodeDlg( IDD_INPUTCODE,"",
                "    ").DoModal() == IDCANCEL)
  {
    delete NewItem;
    return;
  }
  TBtrieve *PBtrFile = new TBtrieve(DivisionDBFileName);
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0) {
    NewItem -> Item.Code = atol(KeyString);
    Status = PBtrFile->Insert((char *)&NewItem->Item, &DataSize,
                              (char *)&Key, 0);
  }
  PBtrFile->Close();
  delete PBtrFile;
  if(Status == 0) {
    NewItem -> Next = NULL;
    if( End != NULL)
      End -> Next = NewItem;
    else
      Home = NewItem;
    End = NewItem;
    SwitchList.AddString(NewItem ->Item.Name);
  }
  else
   MessageBox("      "," ", MB_OK);
}

void CSwitchDlg::DeleteItem( )
{
  int Status = 0;
  DivisionList *Checked;
  long Key;
  int KeyLength = sizeof(Key);
  int DataSize = sizeof(struct Division);

  char Name[MAXDNAMELEN] = "";
  int indx = SwitchList.GetCurSel();
  SwitchList.GetText( indx, Name);
  for( Checked = Home; Checked != NULL; Checked = Checked -> Next)
    if( strcmp(Name, Checked->Item.Name) == 0)
      break;
  if(Checked != NULL) {
    TBtrieve *PBtrFile = new TBtrieve(DivisionDBFileName);
    if((Status = PBtrFile->Open( Owner, omNormal)) == 0) {
      Key = Checked->Item.Code;
      if((Status = PBtrFile->GetEqual((char *)&Key, 0, KeyLength)) == 0) {
        DataSize = sizeof(struct Division);
        Status = PBtrFile->Delete(&DataSize, 0);
      }
    }
    PBtrFile->Close();
    delete PBtrFile;
    if( Status == 0) {
      if( Home == Checked)
        Home = Checked -> Next;
      else {
        DivisionList *Current;
        for( Current = Home; Current -> Next != Checked;
             Current = Current -> Next) ;
        Current -> Next = Checked -> Next;
      }
      SwitchList.DeleteString(indx);
    }
    else
      MessageBox("      "," ", MB_OK);
  }
}

void CSwitchDlg::ListAction( )
{
  char NewString[MAXDNAMELEN];
  SwitchList.GetText( SwitchList.GetCurSel(), NewString);
  SetDlgItemText( IDC_ITEMEDIT, NewString);
}

/*--------------------------------------------------*/
/* CUpdateDlg implementations:                      */
/*--------------------------------------------------*/
CUpdateDlg::CUpdateDlg( UINT Idd, char *DBFileName, char *DTitle)
                 : CDialog(Idd)
{
  DlgDBFileName = new char[strlen(DBFileName) + 1];
  strcpy( DlgDBFileName, DBFileName);
  DlgTitle = new char[strlen(DTitle) + 1];
  strcpy( DlgTitle, DTitle);
//  UpdateList = new CListBox(this, IDC_SWITCHLIST);
}

void CUpdateDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_SWITCHLIST, UpdateList);
}

BOOL CUpdateDlg::OnInitDialog()
{
  CDialog::OnInitDialog();
  ReadFile( DlgDBFileName, (CWnd *) &UpdateList, LBox);
  return TRUE;
}

void CUpdateDlg::AddItem( )
{
  char NewItem[MAXSTRLEN], Key[MAXSTRLEN];
  int DataSize = MAXSTRLEN, Status = 0;
  GetDlgItemText( IDC_ITEMEDIT, NewItem, MAXSTRLEN);
  TBtrieve *PBtrFile = new TBtrieve(DlgDBFileName);
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0) {
    Status = PBtrFile->Insert( NewItem, &DataSize, Key, 0);
  }
  PBtrFile->Close();
  delete PBtrFile;
  if(Status == 0) {
    UpdateList.AddString(NewItem);
  }
  else
   MessageBox("      "," ", MB_OK);
}

void CUpdateDlg::DeleteItem( )
{
  int Status = 0;
  int DataSize = MAXSTRLEN;

  char DelItem[MAXSTRLEN] = "";
  int indx = UpdateList.GetCurSel();
  UpdateList.GetText( indx, DelItem);
  TBtrieve *PBtrFile = new TBtrieve(DlgDBFileName);
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0)
    if((Status = PBtrFile->GetEqual( DelItem, 0, MAXSTRLEN)) == 0) {
      DataSize = MAXSTRLEN;
      Status = PBtrFile->Delete( &DataSize, 0);
    }
    else
      MessageBox("      "," ", MB_OK);
  else
    MessageBox("   "," ", MB_OK);
  PBtrFile->Close();
  delete PBtrFile;
  if( Status == 0)
    UpdateList.DeleteString(indx);
}

void CUpdateDlg::ListAction( )
{
  char NewString[MAXSTRLEN];
  UpdateList.GetText( UpdateList.GetCurSel(), NewString);
  SetDlgItemText( IDC_ITEMEDIT, NewString);
}

/*--------------------------------------------------*/
/* CGeneralViewDlg implementations:                 */
/*--------------------------------------------------*/
CGeneralViewDlg::CGeneralViewDlg(UINT Idd, ViewType Type, long Dvsn)
                 : CDialog(Idd)
{
  Division = Dvsn;
  VType = Type;
//  ViewList = new CListBox(this, IDC_SWITCHLIST);
}

void CGeneralViewDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_SWITCHLIST, ViewList);
}

BOOL CGeneralViewDlg::OnInitDialog()
{
  CBusinesTrip Btrip;
  int DataSize = sizeof(Btrip);
  int Status = 0, KeyNumber;
  long KeyBuf = 0, Key = 0;
  int KeyLength = 0;
  struct Division Work;
  HFONT hLBFont;
  HDC hLBDC;
  WORD HSize;
  WORD wExtent;
  char *Item = new char[300];

  CDialog::OnInitDialog();
  HSize = 0;
  hLBDC = ::GetDC(ViewList.m_hWnd);
  hLBFont = (HFONT) ::SendMessage( ViewList.m_hWnd, WM_GETFONT, 0, 0L);
  if(VType == Debtors) {
	 KeyBuf = Key = 0;
	 KeyNumber = 2;
	 KeyLength = sizeof(unsigned char);
  }
  else
    if(VType == All)
      if(Division == 0) {
	KeyBuf = Key = 1;
	KeyNumber = 0;
	KeyLength = sizeof(unsigned);
      }
      else {
	KeyBuf = Key = Division;
	KeyNumber = 1;
	KeyLength = sizeof(long);
      }
  TBtrieve *BtripFile = new TBtrieve(BtripDBFileName);
  TBtrieve *DvnFile = new TBtrieve(DivisionDBFileName);
  if( BtripFile->Open( Owner, omNormal) == 0 &&
      DvnFile->Open( Owner, omNormal) == 0)
  {
    if(Division != 0) {
      int Size = sizeof(struct Division);
      if(DvnFile->GetEqual((char *)&Work, &Size, (char *)&Division, 0) != 0)
	strcpy( Work.Name,"");
    }
    if((Status = BtripFile->GetEqual((char *)&Btrip, &DataSize,
					  (char *)&KeyBuf, KeyNumber)) == 0)
    {
      while(Status == 0) {
	if(Division == 0) {
	  int Size = sizeof(struct Division);
	  if(DvnFile->GetEqual((char *)&Work, &Size,
			       (char *)&Btrip.Division, 0) != 0)
	    strcpy( Work.Name,"");
	}
	wsprintf(Item,"%20.20s %1.1s %1.1s %35.35s %10.10s %10.10s %20.20s %10.10s %d.%d.%d %d.%d.%d %8.8s ",
		 Btrip.Name, Btrip.ForName, Btrip.SubName, Work.Name,
		 Btrip.City, Btrip.Target, Btrip.Place, Btrip.Trains,
		 (int) Btrip.UDate.Day,(int) Btrip.UDate.Month, Btrip.UDate.Year,
		 (int) Btrip.PDate.Day,(int) Btrip.PDate.Month, Btrip.PDate.Year,
		 Btrip.States);
        wExtent = LOWORD( ::GetTextExtent( hLBDC, Item, strlen(Item)));
	if(wExtent > HSize) HSize = wExtent;
        ViewList.AddString(Item);
	DataSize = sizeof(Btrip);
	Status = BtripFile->GetNext((char *)&Btrip, &DataSize,
					 (char *)&KeyBuf, KeyNumber);
	if( Key != KeyBuf)
	  Status = END_OF_FILE;
      }
      if(Status != END_OF_FILE)
        MessageBox("  "," ", MB_OK);
    }
    else {
      MessageBox("    "," ", MB_OK);
      DestroyWindow();
    }
  }
  else {
    MessageBox("   "," ", MB_OK);
    DestroyWindow();
  }
  BtripFile->Close();
  DvnFile->Close();
  delete BtripFile;
  delete DvnFile;
  ::SendMessage( ViewList.m_hWnd, LB_SETHORIZONTALEXTENT, HSize, 0L);
  ::ReleaseDC( ViewList.m_hWnd, hLBDC);
  return TRUE;
}

/*--------------------------------------------------*/
/* CSelectDlg implementations:                      */
/*--------------------------------------------------*/
CSelectDlg::CSelectDlg( UINT Idd, Exchange *PData)
                 : CDialog(Idd)
{
  Home = NULL;
  PExchangeData = PData;
//  SwitchList = new CListBox(this, IDC_SWITCHLIST);
}

CSelectDlg::~CSelectDlg()
{
  DivisionList *Deleted, *Current;
  for( Current = Home; Current != NULL; ) {
    Deleted = Current;
    Current = Current -> Next;
    delete Deleted;
  }
//  delete SwitchList;
}

void CSelectDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_SWITCHLIST, SwitchList);
}

BOOL CSelectDlg::OnInitDialog()
{
  int DataSize = sizeof(struct Division);
  int Status = 0;
  long Key = 0;
  int KeyLength = sizeof(Key);
  DivisionList *Active, *New = new DivisionList;
  TBtrieve *PBtrFile = new TBtrieve(DivisionDBFileName);

  CDialog::OnInitDialog();
  Active = New;
  Active -> Next = NULL;
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0)
    if((Status = PBtrFile->GetFirst((char *)&Active->Item, &DataSize,
				    (char *)&Key, 0)) == 0) {
      SwitchList.AddString(New->Item.Name);
      Home = Active = New;
      while(Status == 0) {
	New = new DivisionList;
	New -> Next = NULL;
	DataSize = sizeof(struct Division);
	Status = PBtrFile->GetNext((char *)&New->Item, &DataSize,
				   (char *)&Key, 0);
	if(Status == 0) {
	  Active -> Next = New;
          SwitchList.AddString(New->Item.Name);
	  Active = Active -> Next;
	}
	else
	  delete New;
      }
    }
  else
    MessageBox("   "," ", MB_OK);
  PBtrFile->Close();
  delete PBtrFile;
  PExchangeData->Division = 0;
  SwitchList.SetCurSel(0);
  return TRUE;
}

void CSelectDlg::OnOK()
{
  char Name[MAXDNAMELEN];
  int indx = SwitchList.GetCurSel();
  SwitchList.GetText( indx, Name);
  for(DivisionList *Checked = Home; Checked != NULL; Checked = Checked -> Next)
    if( strcmp(Name, Checked->Item.Name) == 0) {
      PExchangeData->Division = Checked->Item.Code;
      break;
    }
  if( Checked == NULL) {
    PExchangeData->Division = 0;
    MessageBox("    "," ", MB_OK);
  }
  CDialog::OnOK();
}

void CSelectDlg::ListAction( )
{
    EndDialog(IDOK);
}

/*--------------------------------------------------*/
/* CSearchDlg implementations:                      */
/*--------------------------------------------------*/
CSearchDlg::CSearchDlg(UINT Idd, Exchange *PData)
             : CDialog(Idd)
{
  PExchangeData = PData;
//  ViewList = new CListBox(this, IDC_SWITCHLIST);
  Division = PData->Division;
  Type = PData->Type;
  for(int i = 0; i < 3; i++) {
    strcpy( Key.Names[i], PData->Names[i]);
    strcpy( KeyBuf.Names[i], PData->Names[i]);
  }
  Home = NULL;
}

CSearchDlg::~CSearchDlg()
{
  BtripList *Deleted, *Current;
  for( Current = Home; Current != NULL; ) {
    Deleted = Current;
    Current = Current -> Next;
    delete Deleted;
  }
//  delete ViewList;
}

void CSearchDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_SWITCHLIST, ViewList);
}

BOOL CSearchDlg::OnInitDialog()
{
  CBusinesTrip Btrip;
  BtripList *Active;
  int DataSize = sizeof(Btrip);
  int Status = 0, KeyNumber;
  struct Division Work;
  HFONT hLBFont;
  HDC hLBDC;
  WORD HSize;
  WORD wExtent;
  char *Item = new char[300];

  CDialog::OnInitDialog();
  HSize = 0;
  hLBDC = ::GetDC( ViewList.m_hWnd);
  hLBFont = (HFONT) ::SendMessage( ViewList.m_hWnd, WM_GETFONT, 0, 0L);
  ::SelectObject( hLBDC, hLBFont);
  if(Type == Prsn) {
    KeyNumber = 4;
  }
  else
    if(Type == Accnt) {
      Key.Sign = 0;
      KeyNumber = 5;
    }
    else
      if(Type == Prpmnt) {
	Key.Sign = 0;
	KeyNumber = 6;
      }
      else { //Type == None;
	*((long *)(&Key)) = Division;
	*((long *)(&KeyBuf)) = Division;
	KeyNumber = 1;
      }
  TBtrieve *BtripFile = new TBtrieve(BtripDBFileName);
  TBtrieve *DvnFile = new TBtrieve(DivisionDBFileName);
  if( BtripFile->Open( Owner, omNormal) == 0 &&
		DvnFile->Open( Owner, omNormal) == 0)
  {
    if(Division != 0) {
      int Size = sizeof(struct Division);
      if(DvnFile->GetEqual((char *)&Work, &Size, (char *)&Division, 0) != 0)
	strcpy( Work.Name,"");
    }
    if((Status = BtripFile->GetEqual((char *)&Btrip, &DataSize,
					  (char *)&KeyBuf, KeyNumber)) == 0)
    {
      int AddIndx;
      Home = Active = new BtripList;
      Active -> Next = NULL;
      while(Status == 0) {
	if(Division == 0) {
	  int Size = sizeof(struct Division);
	  if(DvnFile->GetEqual((char *)&Work, &Size,
					 (char *)&Btrip.Division, 0) != 0)
		 strcpy( Work.Name,"");
	}
	wsprintf(Item,"%20.20s %1.1s %1.1s %35.35s %10.10s %10.10s %20.20s %10.10s %d.%d.%d %d.%d.%d %8.8s ",
		 Btrip.Name, Btrip.ForName, Btrip.SubName, Work.Name,
		 Btrip.City, Btrip.Target, Btrip.Place, Btrip.Trains,
		 (int) Btrip.UDate.Day,(int) Btrip.UDate.Month, Btrip.UDate.Year,
		 (int) Btrip.PDate.Day,(int) Btrip.PDate.Month, Btrip.PDate.Year,
		 Btrip.States);
        wExtent = LOWORD( ::GetTextExtent( hLBDC, Item, strlen(Item)));
	if(wExtent > HSize) HSize = wExtent;
        AddIndx = ViewList.AddString(Item);
	Active -> BtripCode = Btrip.Code;
	Active -> InListIndex = AddIndx;
	Active -> Next = new BtripList;
	Active = Active -> Next;
	Active -> Next = NULL;
	DataSize = sizeof(Btrip);
	Status = BtripFile->GetNext((char *)&Btrip, &DataSize,
				    (char *)&KeyBuf, KeyNumber);
	if( KeyComp() == 0)
	  Status = END_OF_FILE;
      }
      if(Status != END_OF_FILE)
        MessageBox("  "," ", MB_OK);
      else
        ViewList.SetCurSel(0);
    }
    else {
      Home = NULL;
      MessageBox("    "," ", MB_OK);
      DestroyWindow();
    }
  }
  else
    MessageBox("   "," ", MB_OK);
  BtripFile->Close();
  DvnFile->Close();
  delete BtripFile;
  delete DvnFile;
  ::SendMessage( ViewList.m_hWnd, LB_SETHORIZONTALEXTENT, HSize, 0L);
  ::ReleaseDC( ViewList.m_hWnd, hLBDC);
  return TRUE;
}

void CSearchDlg::OnOK()
{
  int SelIndx = ViewList.GetCurSel();
  for(BtripList *Checked = Home; Checked != NULL; Checked = Checked -> Next)
    if( SelIndx == Checked->InListIndex) {
      PExchangeData->Code = Checked->BtripCode;
      break;
    }
  CDialog::OnOK();
}

int CSearchDlg::KeyComp()
{
  switch(Type) {
    case Accnt :
      return (lstrcmp(Key.Names[0],KeyBuf.Names[0]) == 0 &&
	      lstrcmp(Key.Names[1],KeyBuf.Names[1]) == 0 &&
	      lstrcmp(Key.Names[2],KeyBuf.Names[2]) == 0 &&
	      Key.Sign == KeyBuf.Sign);
    case Prpmnt :
      return (lstrcmp(Key.Names[0],KeyBuf.Names[0]) == 0 &&
	      lstrcmp(Key.Names[1],KeyBuf.Names[1]) == 0 &&
	      lstrcmp(Key.Names[2],KeyBuf.Names[2]) == 0 &&
	      Key.Sign == KeyBuf.Sign);
    case Prsn :
      return (lstrcmp(Key.Names[0],KeyBuf.Names[0]) == 0 &&
              lstrcmp(Key.Names[1],KeyBuf.Names[1]) == 0 &&
	      lstrcmp(Key.Names[2],KeyBuf.Names[2]) == 0);
    case None :
      return *((long *)(&KeyBuf)) == Division;
  }
  return 0;
}

/*--------------------------------------------------*/
/* CAttributeDlg implementations:                   */
/*--------------------------------------------------*/
CAttributeDlg::CAttributeDlg( UINT Idd, Exchange *PData)
                : CDialog(Idd)
{
  Home = NULL;
  PExchangeData = PData;
  /*
  KeyGroup = new CButton();
  KeyGroup -> Create( NULL, 0, NULL, IDC_KEYGROUP);
  SearchGroup = new CButton();
  SearchGroup -> Create( NULL, 0, NULL, IDC_SEARCHGROUP);
  DvsnList = new CComboBox();
  DvsnList -> Create( NULL, 0, NULL, IDC_DIVISIONLIST, MAXDNAMELEN);
  Initials = new CButton();
  Initials -> Create( NULL, 0, NULL, IDC_INITIALBUTTON, KeyGroup);
  Dvsns = new CButton();
  Dvsns -> Create( NULL, 0, NULL, IDC_DVSNBUTTON, KeyGroup);
  IsAccount = new CButton();
  IsAccount -> Create( NULL, 0, NULL, IDC_ACCOUNTBUTTON, SearchGroup);
  IsPrep = new CButton();
  IsPrep -> Create( NULL, 0, NULL, IDC_PREPBUTTON, SearchGroup);
  IsPerson = new CButton();
  IsPerson -> Create( NULL, 0, NULL, IDC_PERSONBUTTON, SearchGroup);
  */

  PExchangeData->Division = 0;
  for(int i = 0; i < 3; i++)
    strcpy( PExchangeData->Names[i],"");
}

CAttributeDlg::~CAttributeDlg()
{
  DivisionList *Deleted, *Current;
  for( Current = Home; Current != NULL; ) {
    Deleted = Current;
    Current = Current -> Next;
    delete Deleted;
  }
}

void CAttributeDlg::DoDataExchange( CDataExchange* pDX )
{
  CDialog::DoDataExchange( pDX );
  DDX_Control( pDX, IDC_DIVISIONLIST, DvsnList);
  /*
  DDX_Text( pDX, IDC_NAMEEDIT, PExchangeData->Names[0]);
  DDX_Text( pDX, IDC_FORNAMEEDIT, PExchangeData->Names[1]);
  DDX_Text( pDX, IDC_SUBNAMEEDIT, PExchangeData->Names[2]);
  */
}

BOOL CAttributeDlg::OnInitDialog()
{
  int DataSize = sizeof(struct Division);
  int Status = 0;
  long Key = 0;
  int KeyLength = sizeof(Key);
  DivisionList *Active, *New = new DivisionList;
  TBtrieve *PBtrFile = new TBtrieve(DivisionDBFileName);

  CDialog::OnInitDialog();
  Active = New;
  Active -> Next = NULL;
  if((Status = PBtrFile->Open( Owner, omNormal)) == 0)
    if((Status = PBtrFile->GetFirst((char *)&Active->Item, &DataSize,
				    (char *)&Key, 0)) == 0) {
      DvsnList.AddString(New->Item.Name);
      Home = Active = New;
      while(Status == 0) {
	New = new DivisionList;
	New -> Next = NULL;
	DataSize = sizeof(struct Division);
	Status = PBtrFile->GetNext((char *)&New->Item, &DataSize,
				   (char *)&Key, 0);
	if(Status == 0) {
	  Active -> Next = New;
          DvsnList.AddString(New->Item.Name);
	  Active = Active -> Next;
	}
	else
	  delete New;
      }
    }
  else
    MessageBox("   "," ", MB_OK);
  PBtrFile->Close();
  delete PBtrFile;
  DvsnList.SetCurSel(0);
  PExchangeData->Type = Prsn;
  ForInitials();
  return TRUE;
}

void CAttributeDlg::ForInitials()
{
    //Initials->Check();
    ((CWnd *) GetDlgItem( IDC_NAMEEDIT ))->EnableWindow(TRUE);
    ((CWnd *) GetDlgItem( IDC_FORNAMEEDIT ))->EnableWindow(TRUE);
    ((CWnd *) GetDlgItem( IDC_SUBNAMEEDIT ))->EnableWindow(TRUE);
    ((CWnd *) GetDlgItem( IDC_SEARCHGROUP ))->EnableWindow(TRUE);
    ((CWnd *) GetDlgItem( IDC_ACCOUNTBUTTON ))->EnableWindow(TRUE);
    ((CWnd *) GetDlgItem( IDC_PREPBUTTON ))->EnableWindow(TRUE);
    ((CWnd *) GetDlgItem( IDC_PERSONBUTTON ))->EnableWindow(TRUE);
    //Dvsns->Uncheck();
    ((CWnd *) GetDlgItem( IDC_DIVISIONLIST ))->EnableWindow(FALSE);
}

void CAttributeDlg::ForDvsns()
{
    //Initials->Uncheck();
    ((CWnd *) GetDlgItem( IDC_NAMEEDIT ))->EnableWindow(FALSE);
    ((CWnd *) GetDlgItem( IDC_FORNAMEEDIT ))->EnableWindow(FALSE);
    ((CWnd *) GetDlgItem( IDC_SUBNAMEEDIT ))->EnableWindow(FALSE);
    ((CWnd *) GetDlgItem( IDC_SEARCHGROUP ))->EnableWindow(FALSE);
    ((CWnd *) GetDlgItem( IDC_ACCOUNTBUTTON ))->EnableWindow(FALSE);
    ((CWnd *) GetDlgItem( IDC_PREPBUTTON ))->EnableWindow(FALSE);
    ((CWnd *) GetDlgItem( IDC_PERSONBUTTON ))->EnableWindow(FALSE);
    //Dvsns->Check();
    ((CWnd *) GetDlgItem( IDC_DIVISIONLIST ))->EnableWindow(TRUE);
}

void CAttributeDlg::Account()
{
    PExchangeData->Type = Accnt;
    //IsPrep->Uncheck();
    //IsPerson->Uncheck();
}

void CAttributeDlg::Prepayment()
{
    PExchangeData->Type = Prpmnt;
    //IsAccount->Uncheck();
    //IsPerson->Uncheck();
}

void CAttributeDlg::Person()
{
    PExchangeData->Type = Prsn;
    //IsAccount->Uncheck();
    //IsPrep->Uncheck();
}

void CAttributeDlg::OnOK()
{
  UpdateData();
  if( IsDlgButtonChecked(IDC_INITIALBUTTON) != 0) {
    GetDlgItemText( IDC_NAMEEDIT, PExchangeData->Names[0], MAXNAMELEN);
    GetDlgItemText( IDC_FORNAMEEDIT, PExchangeData->Names[1], MAXNAMELEN);
    GetDlgItemText( IDC_SUBNAMEEDIT, PExchangeData->Names[2], MAXNAMELEN);
    if(! ValidName(PExchangeData->Names[0]) ||
       ! ValidName(PExchangeData->Names[1]) ||
       ! ValidName(PExchangeData->Names[2]))
    {
      MessageBox(" "," ", MB_OK);
      return;
    }
  }
  if( IsDlgButtonChecked(IDC_DVSNBUTTON) != 0) {
    char Name[MAXDNAMELEN] = "";
    DvsnList.GetLBText( DvsnList.GetCurSel(), Name);
    for(DivisionList *Checked = Home; Checked != NULL; Checked = Checked -> Next)
      if( strcmp(Name, Checked->Item.Name) == 0 ) {
	PExchangeData->Division = Checked->Item.Code;
	PExchangeData->Type = None;
	break;
      }
  }
  //PExchangeData->Type = Type;
  CDialog::OnOK();
}

/*--------------------------------------------------*/
/* TCodeDlg implementations:                        */
/*--------------------------------------------------*/
CCodeDlg::CCodeDlg(UINT Idd, LPSTR ATitle, LPSTR APrompt)
                 : CDialog(Idd)
{
  strcpy( DName, ATitle);
  strcpy( DPrompt, ATitle);
}

BOOL CCodeDlg::OnInitDialog()
{
  CDialog::OnInitDialog();
  SetWindowText(DName);
  SetDlgItemText( IDC_PROMPT, DPrompt);
  return TRUE;
}

void CCodeDlg::DoDataExchange( CDataExchange* pDX )
{
  DDX_Control( pDX, IDC_CODE, code);
}

void CCodeDlg::OnOK()
{
  UpdateData(TRUE);
  int len = code.GetWindowTextLength() + 1;
  char *data = new char[len];
  code.GetWindowText( data, len);
  if(! ValidNumeric(data))
    return;
  CDialog::OnOK();
  delete data;
}

BEGIN_MESSAGE_MAP( CSwitchDlg, CDialog )
    ON_BN_CLICKED( IDC_ADDBUTTON, AddItem )
    ON_BN_CLICKED( IDC_DELBUTTON, DeleteItem )
    ON_LBN_SELCHANGE( IDC_SWITCHLIST, ListAction )
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP( CUpdateDlg, CDialog )
    ON_BN_CLICKED( IDC_ADDBUTTON, AddItem )
    ON_BN_CLICKED( IDC_DELBUTTON, DeleteItem )
    ON_LBN_SELCHANGE( IDC_SWITCHLIST, ListAction )
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP( CSelectDlg, CDialog )
    ON_LBN_DBLCLK( IDC_SWITCHLIST, ListAction )
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP( CAttributeDlg, CDialog )
    ON_BN_CLICKED( IDC_ACCOUNTBUTTON, Account )
    ON_BN_CLICKED( IDC_PREPBUTTON, Prepayment )
    ON_BN_CLICKED( IDC_PERSONBUTTON, Person )
    ON_BN_CLICKED( IDC_INITIALBUTTON, ForInitials )
    ON_BN_CLICKED( IDC_DVSNBUTTON, ForDvsns )
END_MESSAGE_MAP()
