#include <mem.h>
#include <string.h>
#include "tbtrieve.h"

#ifdef __BORLANDC__
#ifdef _Windows
#define __Win16__
#endif
#endif

#ifdef __WATCOMC__
#ifdef __WINDOWS__
#define __Win16__
#endif
#endif

#ifdef __WATCOMC__
#pragma aux BTRV parm []
#endif

#ifdef __Win16__
extern "C" { int BTRVCALL( int  OP, char far *POS_BLK,
                                char far *DATA_BUF, int far *DATA_LEN,
                                char far *KEY_BUF, int  KEY_NUM, int  KEY_LEN); };
#else
extern "C" { int BTRVCALL( int  OP, char far *POS_BLK,
                                char far *DATA_BUF, int far *DATA_LEN,
                                char far *KEY_BUF, int  KEY_NUM); };
#endif

int BTRV( int  OP, char far *POS_BLK, char far *DATA_BUF, int far *DATA_LEN,
          char far *KEY_BUF, int  KEY_NUM, int KEY_LEN = 0)
{
#ifdef __Win16__
  return BTRVCALL( OP, POS_BLK, DATA_BUF, DATA_LEN, KEY_BUF, KEY_NUM, KEY_LEN);
#else
  return BTRVCALL( OP, POS_BLK, DATA_BUF, DATA_LEN, KEY_NUM, KEY_BUF);
#endif
}

//*** ABORT TRANSACTION

TBtrieve::TBtrieve(char *BtrieveFileName)
{
  FileName = new char [strlen(BtrieveFileName) + 1];
  strcpy( FileName, BtrieveFileName);
  Lock = 0;
  Status = 0;
  IsOpen = 0;
  memset((void *)PosBlock, 0, sizeof(PosBlock));
};

int TBtrieve::AbortTransaction(void)
{
   Status = BTRV( 21, NULL, NULL, NULL, NULL, 0 ); return Status;
}

//*** BEGIN TRANSACTION

int TBtrieve::BeginTransaction(void)
{
   Status = BTRV( 19 + Lock, NULL, NULL, NULL, NULL, 0 ); return Status;
}

//*** CLEAR OWNER

int TBtrieve::ClearOwner(void)
{
   Status = BTRV( 30, PosBlock, NULL, NULL, NULL, 0 ); return Status;
}

//*** CLOSE

int TBtrieve::Close(void)
{
   Status = BTRV( 1, PosBlock, NULL, NULL, NULL, 0 );
   if (Status == 0) IsOpen = 0;
   return Status;
}

//*** CREATE SUPPLEMENTAL INDEX

int TBtrieve::CreateSupplementalIndex( struct TKeySpecs KeySpecs[],
                                       int KeySpecsLength )
{
  Status = BTRV( 31, PosBlock, (char *)KeySpecs, &KeySpecsLength, NULL, 0 );
  return Status;
}

//*** DELETE

int TBtrieve::Delete( int *RecordLength,  // io
                      int KeyNumber )      // i
{
  Status = BTRV( 4, PosBlock, NULL, RecordLength, NULL, KeyNumber );
  return Status;
}

int TBtrieve::Delete( int KeyNumber )      // i
{
  int RecordLength = Attributes.RecordLength;
  Status = BTRV( 4, PosBlock, NULL, &RecordLength, NULL, KeyNumber );
  return Status;
}

//*** DROP SUPPLEMENTAL INDEX

int TBtrieve::DropSupplementalIndex( int KeyNumber )   // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 32, PosBlock, NULL, NULL, NULL, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** END TRANSACTION

int TBtrieve::EndTransaction(void)
{
   Status = BTRV( 20, NULL, NULL, NULL, NULL, 0 ); return Status;
}

//*** EXTEND

int TBtrieve::Extend( char ExtensionFileName[],    // i
                      int  UseExtensionFile )      // i
{
   Status = BTRV( 16, PosBlock, NULL, NULL, ExtensionFileName,
                  UseExtensionFile );
   return Status;
}

//*** GET DIRECT

int TBtrieve::GetDirect( long Position,      // i
                         char Record[],      //  o
                         int *RecordLength,  // io
                         char Key[],         //  o
                         int  KeyNumber)     // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  memmove( &Position, Record, 4 );
  Status = BTRV( 23 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetDirect( long Position,      // i
                         char Record[],      //  o
                         char Key[],         //  o
                         int  KeyNumber)     // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  memmove( &Position, Record, 4 );
  Status = BTRV( 23 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//*** GET DIRECTORY

int TBtrieve::GetDirectory( char CurrentDirectory[], //  o
                            int  DriveNumber )       // i
{
  Status = BTRV( 18, NULL, NULL, NULL, CurrentDirectory, DriveNumber );
  return Status;
}

//*** GET EQUAL

int TBtrieve::GetEqual( char Record[],      //  o
                        int *RecordLength,  // io
                        char Key[],         // i
                        int  KeyNumber)     // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 5 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetEqual( char Record[],      //  o
                        char Key[],         // i
                        int  KeyNumber)     // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 5 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetEqual( char Key[],         // i
                        int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 5 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//***  GET FIRST

int TBtrieve::GetFirst( char Record[],      //  o
                        int *RecordLength,  // io
                        char Key[],         //  o
                        int  KeyNumber)     // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 12 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetFirst( char Record[],      //  o
                        char Key[],         //  o
                        int  KeyNumber)     // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 12 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetFirst( char Key[],         //  o
                        int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 12 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET GREATER

int TBtrieve::GetGreater( char Record[],      //  o
                          int *RecordLength,  // io
                          char Key[],         // io
                          int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 8 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetGreater( char Record[],      //  o
                          char Key[],         // io
                          int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 8 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetGreater( char Key[],         // io
                          int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 8 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET GREATER OR EQUAL

int TBtrieve::GetGreaterOrEqual( char Record[],      //  o
                                 int *RecordLength,  // io
                                 char Key[],         // io
                                 int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 9 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetGreaterOrEqual( char Record[],      //  o
                                 char Key[],         // io
                                 int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 9 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetGreaterOrEqual( char Key[],         // io
                                 int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 9 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET LAST

int TBtrieve::GetLast( char Record[],      //  o
                       int *RecordLength,  // io
                       char Key[],         //  o
                       int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 13 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetLast( char Record[],      //  o
                       char Key[],         //  o
                       int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 13 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetLast( char Key[],         //  o
                       int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 13 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET LESS THAN

int TBtrieve::GetLessThan( char Record[],      //  o
                           int *RecordLength,  // io
                           char Key[],         // io
                           int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 10 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetLessThan( char Record[],      //  o
                           char Key[],         // io
                           int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 10 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetLessThan( char Key[],         // io
                           int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 10 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET LESS THAN OR EQUAL

int TBtrieve::GetLessThanOrEqual( char Record[],      //  o
                                  int *RecordLength,  // io
                                  char Key[],         // io
                                  int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 11 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetLessThanOrEqual( char Record[],      //  o
                                  char Key[],         // io
                                  int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 11 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetLessThanOrEqual( char Key[],         // io
                                  int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 11 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET NEXT

int TBtrieve::GetNext( char Record[],      //  o
                       int *RecordLength,  // io
                       char Key[],         // io
                       int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 6 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetNext( char Record[],      //  o
                       char Key[],         // io
                       int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 6 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetNext( char Key[],         // io
                       int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 6 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** GET POSITION

int TBtrieve::GetPosition( long *Position)    //  o
{
  static int Length = 4;
  Status = BTRV( 22, PosBlock, ( char *)Position, &Length, NULL, 0 );
  return Status;
}

//*** GET PREVIOUS

int TBtrieve::GetPrevious( char Record[],      //  o
                           int *RecordLength,  // io
                           char Key[],         // io
                           int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 7 + Lock, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::GetPrevious( char Record[],      //  o
                           char Key[],         // io
                           int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 7 + Lock, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

int TBtrieve::GetPrevious( char Key[],         // io
                           int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 7 + 50 + Lock, PosBlock, NULL, NULL, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

//*** INSERT

int TBtrieve::Insert( char Record[],      // i
                      int *RecordLength,  // i
                      char Key[],         // o
                      int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 2, PosBlock, Record, RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  return Status;
}

int TBtrieve::Insert( char Record[],      // i
                      char Key[],         // o
                                int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 2, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//***  LOCKS

void TBtrieve::Locks(int LockType)
{
  Lock = LockType;
}

//*** OPEN

int TBtrieve::Open( char OwnerName[], // i
                         int  Mode )       // i
{
  char ExtentionFileName[64];
  static int Length = 1 + strlen( OwnerName );
  int AttrLenght = sizeof(Attributes);
  Status = BTRV( 0 + Lock, PosBlock, OwnerName, &Length, FileName, Mode
#ifdef __Win16__
  , strlen(FileName)
#endif
  );
  if( Status != 0) {
    IsOpen = 0;
    return Status;
  }
  Status = Stat( &Attributes, &AttrLenght, ExtentionFileName);
  if( Status == 0) IsOpen = 1;
  else IsOpen = 0;
  return Status;
}

//***  RESET

int TBtrieve::Reset( int ConnectionNumber,  // i
                     int Mode )             // i
{
  Status = BTRV( 28, NULL, NULL, NULL,( char * ) &ConnectionNumber, Mode );
  return Status;
}

//*** SET DIRECTORY

int TBtrieve::SetDirectory( char Directory[] )   // i
{
  Status = BTRV( 17, NULL, NULL, NULL, Directory, 0 ); return Status;
}

//*** SET OWNER

int TBtrieve::SetOwner( char OwnerName[],           // i
                        int  AccessRestrictions )   // i
{
  int L = strlen( OwnerName );
  Status = BTRV( 29, PosBlock, OwnerName, &L, OwnerName, AccessRestrictions );
  return Status;
}

//*** STAT

int TBtrieve::Stat( struct TFileSpecs *FileSpecs, // io
                    int *FileSpecsLength,         // i
                    char *ExtensionFileName )     // o
{
  Status = BTRV( 15, PosBlock, (char *) FileSpecs, FileSpecsLength,
                 ExtensionFileName, 0 );
  return Status;
}

//*** STEP FIRST

int TBtrieve::StepFirst( char Record[],        //  o
                         int *RecordLength )   // io
{
  Status = BTRV( 33 + Lock, PosBlock, Record, RecordLength, NULL, 0 );
  return Status;
}

int TBtrieve::StepFirst( char Record[] )   // o
{
  int RecordLength = Attributes.RecordLength;
  Status = BTRV( 33 + Lock, PosBlock, Record, &RecordLength, NULL, 0 );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//*** STEP LAST

int TBtrieve::StepLast( char Record[],        //  o
                        int *RecordLength )   // io
{
  Status = BTRV( 34 + Lock, PosBlock, Record, RecordLength, NULL, 0 );
  return Status;
}

int TBtrieve::StepLast( char Record[] )   // o
{
  int RecordLength = Attributes.RecordLength;
  Status = BTRV( 34 + Lock, PosBlock, Record, &RecordLength, NULL, 0 );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//*** STEP NEXT

int TBtrieve::StepNext( char Record[],        //  o
                        int *RecordLength )   // io
{
  Status = BTRV( 24 + Lock, PosBlock, Record, RecordLength, NULL, 0 );
  return Status;
}

int TBtrieve::StepNext( char Record[] )       // o
{
  int RecordLength = Attributes.RecordLength;
  Status = BTRV( 24 + Lock, PosBlock, Record, &RecordLength, NULL, 0 );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//*** STEP PREVIOUS

int TBtrieve::StepPrevious( char Record[],        //  o
                            int *RecordLength )   // io
{
  Status = BTRV( 35 + Lock, PosBlock, Record, RecordLength, NULL, 0 );
  return Status;
}

int TBtrieve::StepPrevious( char Record[] )       // o
{
  int RecordLength = Attributes.RecordLength;
  Status = BTRV( 35 + Lock, PosBlock, Record, &RecordLength, NULL, 0 );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//*** STOP

int TBtrieve::Stop(void)
{
   Status = BTRV( 25, NULL, NULL, NULL, NULL, 0 );
   return Status;
}

//*** UNLOCK

int TBtrieve::Unlock( long *Position,      // i
                      int  UnlockType )   // i
{
   int L = 4;
   Status = BTRV( 27, PosBlock, ( char * ) Position, &L, NULL, UnlockType );
   return Status;
}

//*** UPDATE

int TBtrieve::Update( char Record[],      // i
                      int *RecordLength,  // i
                      char Key[],         //  o
                      int  KeyNumber )    // i
{
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
   Status = BTRV( 3, PosBlock, Record, RecordLength, Key, KeyNumber,
                  Attributes.KeySpecs[KeyNumber].KeyLength );
   return Status;
}

int TBtrieve::Update( char Record[],      // i
                      char Key[],         //  o
                      int  KeyNumber )    // i
{
  int RecordLength = Attributes.RecordLength;
  if( KeyNumber > Attributes.NumberOfIndexes - 1)
    return (Status = INVALID_KEY_NUMBER);
  Status = BTRV( 3, PosBlock, Record, &RecordLength, Key, KeyNumber,
                 Attributes.KeySpecs[KeyNumber].KeyLength );
  if( RecordLength != Attributes.RecordLength)
    Status = DATA_BUFFER_LENGTH;
  return Status;
}

//*** VERSION

int TBtrieve::Version( int  *Version,    //  o
                       int  *Revision,   //  o
                       char *NetWare )   //  o
{
   struct {
     int  V;
     int  R;
     char N;
   } Record;
   int L = 5;

   Status = BTRV( 26, NULL, ( char * ) &Record, &L, NULL, 0 );

   *Version = Record.V;
   *Revision = Record.R;
   *NetWare = Record.N;

   return Status;
}

//*** CHANGE FILE
int TBtrieve::ChangeFile( char *NewFileName )
{
  if (IsOpen) Close();
  if (Status != 0) return 1; // Error
  delete FileName;
  FileName = new char [strlen(NewFileName) + 1];
  strcpy(FileName, NewFileName);
  Lock = 0;
  Status = 0;
  IsOpen = 0;
  memset((void *) PosBlock, 0, sizeof(PosBlock));
  return 1;
}

//*** CREATE

int BtrieveCreate( char PosBlock[],
                   struct TFileSpecs FileSpecs,
                   int * FileSpecsLength,
                   char FileName[],
                   int Mode )
{
  return ( BTRV( 14, PosBlock, (char *)&FileSpecs, FileSpecsLength,
                 FileName, Mode ) );
}
