// BTRIEVE STATUS CODES

#define OK                                      0
#define INVALID_OPERATION                       1
#define IO_ERROR                                2
#define FILE_NOT_OPEN                           3
#define KEY_VALUE_NOT_FOUND                     4
#define DUPLICATE_KEY_VALUE                     5
#define INVALID_KEY_NUMBER                      6
#define DIFFERENT_KEY_NUMBER                    7
#define INVALID_POSITIONING                     8
#define END_OF_FILE                             9
#define MODIFIABLE_KEY_VALUE_ERROR             10
#define INVALID_FILE_NAME                      11
#define FILE_NOT_FOUND                         12
#define EXTENDED_FILE_ERROR                    13
#define PRE_IMAGE_OPEN_ERROR                   14
#define PRE_IMAGE_IO_ERROR                     15
#define EXPANSION_ERROR                        16
#define CLOSE_ERROR                            17
#define DISK_FULL                              18
#define UNRECOVERABLE_ERROR                    19
#define RECORD_MANAGER_INACTIVE                20
#define KEY_BUFFER_TOO_SHORT                   21
#define DATA_BUFFER_LENGTH                     22
#define POSITION_BLOCK_LENGTH                  23
#define PAGE_SIZE_ERROR                        24
#define CREATE_IO_ERROR                        25
#define NUMBER_OF_KEYS                         26
#define INVALID_KEY_POSITION                   27
#define INVALID_RECORD_LENGTH                  28
#define INVALID_KEY_LENGTH                     29
#define NOT_A_BTRIEVE_FILE                     30
#define FILE_ALREDY_EXTENDED                   31
#define EXTEND_IO_ERROR                        32
#define INVALID_EXTENSION_NAME                 34
#define DIRECTORY_ERROR                        35
#define TRANSACTION_ERROR                      36
#define TRANSACTION_IS_ACTIVE                  37
#define TRANSACTION_CONTROL_FILE_IO_ERROR      38
#define END_ABORT_TRANSACTION_ERROR            39
#define TRANSACTION_MAX_FILES                  40
#define OPERATION_NOT_ALLOWED                  41
#define INCOMPLETE_ACCELERATED_ACCESS          42
#define INVALID_RECORD_ADDRESS                 43
#define NULL_KEY_PATH                          44
#define INCONSISTENT_KEY_FLAGS                 45
#define ACCESS_TO_FILE_DENIED                  46
#define MAXIMUM_OPEN_FILES                     47
#define INVALID_ALTERNATE_SEQUENCE_DEFINITION  48
#define KEY_TYPE_ERROR                         49
#define OWNER_ALREADY_SET                      50
#define INVALID_OWNER                          51
#define ERROR_WRITING_CACHE                    52
#define INVALID_INTERFACE                      53
#define VARIALE_PAGE_ERROR                     54
#define AUTOINCREMENT_ERROR                    55
#define INCOMPLETE_INDEX                       56
#define COMPRESSION_BUFFER_TOO_SHORT           58
#define FILE_ALREADY_EXIST                     59
#define CONFLICT                               80
#define LOCK_ERROR                             81
#define LOST_POSITION                          82
#define READ_OUTSIDE_TRANSACTION               83
#define RECORD_IN_USE                          84
#define FILE_IN_USE                            85
#define FILE_TABLE_FULL                        86
#define HANDLE_TABLE_FULL                      87
#define INCOMPATIBLE_MODE_ERROR                88
#define REDIRECTED_DEVICE_TABLE_FULL           90
#define SERVER_ERROR                           91
#define TRANSACTION_TABLE_FULL                 92
#define INCOMPATIBLE_LOCK_TYPE                 93
#define PERMISSION_ERROR                       94
#define SESSION_NOT_LONGER_VALID               95
#define COMMUNICATIONS_ENVIRONMENT_ERROR       96
#define DATA_MESSAGE_TOO_SMALL                 97
#define INTERNAL_TRANSACTION_ERROR             98

// BREQUEST STATUS CODES (OS/2)
#define INSUFFICIENT_MEMORY                  2001
#define PARAMETER_INVALID_OR_OUT_OF_RANGE    2002
#define NO_LOCAL_ACCESS_ALLOWED              2003

//** CREATE MODE
#define cmOverwrite     0
#define cmNoOverwrite  -1
//**

struct TKeySpecs
 { int KeyPosition;
   int KeyLength;
   int KeyFlags;
   long NumberOfKeys;
   char ExtendedKeyType;
   char NullValue;
   long Reserved; };

//** KEY FLAGS
#define kfDuplicate      1
#define kfModifiable     2
#define kfBinary         4
#define kfNull           8
#define kfSegmented     16
#define kfAltColSeq     32
#define kfDescending    64
#define kfSupplemental 128
#define kfExtendedType 256
#define kfManual       512
//**

//** EXTENDED KEY TYPES
#define etString          0
#define etInteger         1
#define etFloat           2
#define etDate            3
#define etTime            4
#define etDecimal         5
#define etMoney           6
#define etLogical         7
#define etNumeric         8
#define etBfloat          9
#define etLstring        10
#define etZstring        11
#define etUnsignedBinary 14
#define etAutoincrement  15
//**

struct TFileSpecs
 { int RecordLength;
   int PageSize;
   int NumberOfIndexes;
   long NumberOfRecords;
   int FileFlags;
   int ReservedWord;
   int Allocation;
   struct TKeySpecs KeySpecs[24];
   char AltColSec[265]; };

//** FILE FLAGS
#define ffVariableLength  1
#define ffBlankTruncation 2
#define ffPreallocation   4
#define ffDataCompression 8
#define ffKeyOnly         16
#define ff_10pcFreeSpace  64
#define ff_20pcFreeSpace  128
#define ff_30pcFreeSpace  192
//**

int BtrieveCreate( char PosBlock[],
                   struct TFileSpecs FileSpecs,
                   int * FileSpecsLength,
                   char FileName[],
                   int Mode );

//** USE EXTENSION FILE
#define ueUseImmediate   -1
#define ueNoUseImmediate  0

//***  LOCK TYPE

#define ltNoLock                 0
#define ltSingleRecordWait     100
#define ltSingleRecordNoWait   200
#define ltMultipleRecordWait   300
#define ltMultipleRecordNoWait 400

//** Open Mode

#define omAccelerated -1
#define omReadOnly    -2
#define omVerify      -3
#define omExclusive   -4
#define omNormal       0

//** Reset Mode

#define rmForAnother  -1
#define rmForYourSelf  0

//** ACCESS RESTRICTIONS
#define arOwner                0
#define arOtherReadOnly        1
#define arOwnerEncrypt         2
#define arOtherReadOnlyEncrypt 3
//**

//** UNLOCK TYPE

#define utSingle       0
#define utOneMultiple -1
#define utAllMultiple -2

class TBtrieve {
private:
  char *FileName;
  struct TFileSpecs Attributes;
  char PosBlock[128];
  int Lock;
  int Status;
  int IsOpen;

public:
//constructor
  TBtrieve(char *BtrieveFileName);
//destructor
  ~TBtrieve(void) { delete FileName; };
//*** OPEN
  int Open( char OwnerName[], int  Mode );

//*** ABORT TRANSACTION
  int AbortTransaction(void);

//*** BEGIN TRANSACTION
  int BeginTransaction(void);

//*** CLEAR OWNER
  int ClearOwner(void);  // io

//*** CLOSE
  int Close(void);  // i

//*** CREATE SUPPLEMENTAL INDEX
  int CreateSupplementalIndex( struct TKeySpecs KeySpecs[], // i
                               int KeySpecsLength );        // i

//*** DELETE
  int Delete( int *RecordLength,  // io
              int KeyNumber );    // i

  int Delete( int KeyNumber );    // i

//*** DROP SUPPLEMENTAL INDEX
  int DropSupplementalIndex( int KeyNumber);  // i

//*** END TRANSACTION
  int EndTransaction(void);

//*** EXTEND
  int Extend( char ExtensionFileName[],    // i
              int  UseExtensionFile );     // i

//*** GET DIRECT
  int GetDirect( long Position,      // i
                 char Record[],      // o
                 int *RecordLength,  // io
                 char Key[],         // o
                 int  KeyNumber);   // i

  int GetDirect( long Position,      // i
                 char Record[],      // o
                 char Key[],         // o
                 int  KeyNumber);   // i

//*** GET DIRECTORY
  int GetDirectory( char CurrentDirectory[], //  o
                    int  DriveNumber );      // i

//*** GET EQUAL
  int GetEqual( char Record[],      //  o
                int *RecordLength,  // io
                char Key[],         // i
                int  KeyNumber);    // i

  int GetEqual( char Record[],      //  o
                char Key[],         // i
                int  KeyNumber);    // i

  int GetEqual( char Key[],         // i
                int  KeyNumber);    // i


//***  GET FIRST
  int GetFirst( char Record[],      //  o
                int *RecordLength,  // io
                char Key[],         //  o
                int  KeyNumber);    // i

  int GetFirst( char Record[],      //  o
                char Key[],         //  o
                int  KeyNumber);    // i

  int GetFirst( char Key[],         //  o
                int  KeyNumber);    // i

//*** GET GREATER
  int GetGreater( char Record[],      //  o
                  int *RecordLength,  // io
                  char Key[],         // io
                  int  KeyNumber);    // i

  int GetGreater( char Record[],      //  o
                  char Key[],         // io
                  int  KeyNumber);    // i

  int GetGreater( char Key[],         // io
                  int  KeyNumber);    // i

//*** GET GREATER OR EQUAL
  int GetGreaterOrEqual( char Record[],      //  o
                         int *RecordLength,  // io
                         char Key[],         // io
                         int  KeyNumber);    // i

  int GetGreaterOrEqual( char Record[],      //  o
                         char Key[],         // io
                         int  KeyNumber);    // i

  int GetGreaterOrEqual( char Key[],         // io
                         int  KeyNumber);    // i

//*** GET LAST
  int GetLast( char Record[],      //  o
               int *RecordLength,  // io
               char Key[],         //  o
               int  KeyNumber);    // i

  int GetLast( char Record[],      //  o
               char Key[],         //  o
               int  KeyNumber);    // i

  int GetLast( char Key[],         //  o
               int  KeyNumber);    // i

//*** GET LESS THAN
  int GetLessThan( char Record[],      //  o
                   int *RecordLength,  // io
                   char Key[],         // io
                   int  KeyNumber);    // i

  int GetLessThan( char Record[],      //  o
                   char Key[],         // io
                   int  KeyNumber);    // i


  int GetLessThan( char Key[],         // io
                   int  KeyNumber);    // i

//*** GET LESS THAN OR EQUAL
  int GetLessThanOrEqual( char Record[],      //  o
                          int *RecordLength,  // io
                          char Key[],         // io
                          int  KeyNumber);    // i

  int GetLessThanOrEqual( char Record[],      //  o
                          char Key[],         // io
                          int  KeyNumber);    // i

  int GetLessThanOrEqual( char Key[],         // io
                          int  KeyNumber);    // i

//*** GET NEXT
  int GetNext( char Record[],      //  o
               int *RecordLength,  // io
               char Key[],         // io
               int  KeyNumber);    // i

  int GetNext( char Record[],      //  o
               char Key[],         // io
               int  KeyNumber);    // i

  int GetNext( char Key[],         // io
               int  KeyNumber);    // i

//*** GET POSITION
  int GetPosition( long *Position );

//*** GET PREVIOUS
  int GetPrevious( char Record[],      //  o
                   int *RecordLength,  // io
                   char Key[],         // io
                   int  KeyNumber);    // i

  int GetPrevious( char Record[],      //  o
                   char Key[],         // io
                   int  KeyNumber);    // i

  int GetPrevious( char Key[],         // io
                   int  KeyNumber);    // i

//*** INSERT
  int Insert( char Record[],      // i
              int  *RecordLength, // i
              char Key[],         //  o
              int  KeyNumber);    // i

  int Insert( char Record[],      // i
              char Key[],         //  o
              int  KeyNumber);    // i

//*** SET LOCKS TYPE
  void Locks( int LockType);

//***  RESET
  int Reset( int ConnectionNumber, int Mode );

//*** SET DIRECTORY
  int SetDirectory( char Directory[] );

//*** SET OWNER
  int SetOwner( char OwnerName[], int  AccessRestrictions );

//*** STAT
  int Stat( struct TFileSpecs *FileSpecs, int *FileSpecsLength,
            char ExtensionFileName[64] );

//*** STEP FIRST
  int StepFirst( char Record[],           // io
		 int *RecordLength );     // io

  int StepFirst( char Record[]);          // io

//*** STEP LAST
  int StepLast( char Record[],		  // io
		int *RecordLength );      // io

  int StepLast( char Record[]);	  	  // io

//*** STEP NEXT
  int StepNext( char Record[],		  // io
		int *RecordLength );      // io

  int StepNext( char Record[]);		  // io

//*** STEP PREVIOUS
  int StepPrevious( char Record[],	  // io
		    int *RecordLength );  // io

  int StepPrevious( char Record[]);	  // io

//*** STOP
  int Stop(void);

//*** UNLOCK
  int Unlock( long *Position, int  UnlockType );

//*** UPDATE
  int Update( char Record[],      // i
	      int *RecordLength,  // i
	      char Key[],         //  o
	      int  KeyNumber);    // i

  int Update( char Record[],      // i
	      char Key[],         //  o
	      int  KeyNumber);    // i

//*** VERSION
  int Version( int  *Version, int  *Revision, char *NetWare );

//*** GHANGE FILE
  int ChangeFile( char *NewFileName );

//*** GET STATUS OF PREVIOUS OPERATION
  int GetOperationStatus(void) { return Status;};

}; // End Of TBtrieve Class

